VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CPieChart"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'Dieser Source stammt von http://www.activevb.de
'und kann frei verwendet werden. Fr eventuelle Schden
'wird nicht gehaftet.
'
'Um Fehler oder Fragen zu klren, nutzen Sie bitte unser Forum.
'Ansonsten viel Spa und Erfolg mit diesem Source !
'

'*********************************
'* CPieChart.cls                 *
'* draws a pie chart             *
'* Programmed: Achim Neubauer    *
'* Last Change: 14.09.2003 14:25 *
'* Version: 1.0.1                *
'*********************************

Option Explicit

'*** gdi function declarations

Private Declare Function CreatePen Lib "gdi32.dll" (ByVal nPenStyle As Long, ByVal nWidth As Long, ByVal crColor As Long) As Long
Private Declare Function SelectObject Lib "gdi32.dll" (ByVal hDC As Long, ByVal hObject As Long) As Long
Private Declare Function DeleteObject Lib "gdi32.dll" (ByVal hObject As Long) As Long
Private Declare Function CreateSolidBrush Lib "gdi32.dll" (ByVal crColor As Long) As Long

Private Declare Function Pie Lib "gdi32" (ByVal hDC As Long, ByVal nLeftRect As Long, ByVal nTopRect As Long, ByVal nRightRect As Long, ByVal nBottomRect As Long, ByVal nXRadial1 As Long, ByVal nYRadial1 As Long, ByVal nXRadial2 As Long, ByVal nYRadial2 As Long) As Long
Private Declare Function Ellipse Lib "gdi32" (ByVal hDC As Long, ByVal X1 As Long, ByVal Y1 As Long, ByVal X2 As Long, ByVal Y2 As Long) As Long
Private Declare Function LineTo Lib "gdi32.dll" (ByVal hDC As Long, ByVal X As Long, ByVal Y As Long) As Long

'*** class properties

Public Max As Long 'returns the sum of the segment values
Public ClockWise As Boolean 'true = clock wise; false (default) = counter clock wise

Public Enum AngleSystemConstants
  asRAD
  asDEG
End Enum
'*** some internal data stuff

Private Type Segment
  Value As Long
  Color As Long
  Visible As Boolean
  Distant As Boolean
End Type

Private Type DBLPOINT
  X As Double
  Y As Double
End Type

Private Segments() As Segment
Private nSegments As Long

Private m_Angle As Double
Private m_Depth As Long
Private m_SmallDistance As Long
Private m_LargeDistance As Long

Private Const Pi As Double = 3.14159265358979

'*** class methods

'----------------------------------------------------
' Add
' adds a segment and returns the index of the new segment
' - IN Value as Long: absolute value of the segment.
' - IN Color As Long: (optional) color of the segment; if not given, random color is used
' - IN Visible As Boolean: (optional) specifies whether segment is visible; if not given, segment will be visible
' - IN Index As Long: optional index of the segment; if not given, new item will be appended
'----------------------------------------------------
Public Function Add(ByVal Value As Long, Color As Long, Optional Visible As Boolean = True, Optional Distant As Boolean = False, Optional ByVal Index As Long) As Long
  Dim T As Long
  
  If Value < 1 Then Exit Function
  
  If Index < 1 Or Index > nSegments Then
    Index = nSegments + 1
  End If
  
  ReDim Preserve Segments(nSegments + 1)
  nSegments = nSegments + 1
  
  For T = nSegments To Index + 1 Step -1 'Elemente verschieben
    Segments(T) = Segments(T - 1)
  Next T
  
  Segments(Index).Color = Color
  Segments(Index).Value = Value
  Segments(Index).Visible = Visible
  Segments(Index).Distant = Distant
  
  Add = Index
  
  Call CheckMax
End Function

'----------------------------------------------------
' Remove
' removes a segment
' - IN Index As Long: index of segment to be removed
'----------------------------------------------------
Public Sub Remove(ByVal Index As Long)
  Dim T As Long
  
  If Index > 0 And Index <= nSegments Then
    For T = Index To UBound(Segments) - 1 'Elemente verschieben
      Segments(T) = Segments(T + 1)
    Next T
  
    ReDim Preserve Segments(nSegments - 1)
    nSegments = nSegments - 1

    Call CheckMax
  End If
End Sub

'----------------------------------------------------
' Clear
' removes all segments
'----------------------------------------------------
Public Sub Clear()
  nSegments = 0
  ReDim Segments(0)
  
  Max = 0
End Sub

'----------------------------------------------------
' Count
' retrieves the number of segments
'----------------------------------------------------
Public Function Count() As Long
  Count = nSegments
End Function

'----------------------------------------------------
' Paint
' paints a segment
' - IN hDC as Long: handle to a device context
' - IN X as Long: x-coordinate, in pixel, of the upper-left edge
' - IN Y As Long: y-coordinate, in pixel, of the upper-left edge
' - IN Width As Long: width, in pixel, of the chart
' - IN Height As Long: height, in pixel, of the chart
'----------------------------------------------------
Public Sub Paint(ByVal hDC As Long, ByVal X As Long, ByVal Y As Long, ByVal Width As Long, ByVal Height As Long)
  Dim T As Long
  Dim N As Long
  
  Dim Radius As DBLPOINT
  
  Dim SegmentCount As Long
  
  For T = 1 To nSegments
    If Segments(T).Value > 0 Then SegmentCount = SegmentCount + 1
  Next T
  
  Select Case SegmentCount
    Case 1
      X = X
      Y = Y + m_Depth
      Width = Width
      Height = Height - m_Depth
    Case Is > 1
      X = X + m_LargeDistance
      Y = Y + m_LargeDistance + m_Depth
      Width = Width - 2 * m_LargeDistance
      Height = Height - 2 * m_LargeDistance - m_Depth
  End Select
  
  Radius.X = Width / 2
  Radius.Y = Height / 2
  
  Dim FromOffset As Long
  Dim ToOffset As Long
  
  Dim FromWinkel As Double
  Dim ToWinkel As Double
  
  Dim StartPunkt As DBLPOINT
  Dim EndPunkt As DBLPOINT
  
  Dim Drift As DBLPOINT
  
  Dim hPen As Long
  Dim hOldPen As Long
  
  Dim hBrush As Long
  Dim hOldBrush As Long
  
  If hDC <> 0 Then
    Select Case SegmentCount
      Case 1
        For N = 1 To Depth
          If Segments(1).Visible Then
            hPen = CreatePen(vbSolid, 1, MakeDarkColor(Segments(1).Color))
            hOldPen = SelectObject(hDC, hPen)
            
            hBrush = CreateSolidBrush(Segments(1).Color)
            hOldBrush = SelectObject(hDC, hBrush)
            
            Call Ellipse(hDC, X, Y - N, X + Width, Y + Height - N)
            
            Call SelectObject(hDC, hOldBrush)
            Call SelectObject(hDC, hOldPen)
            Call DeleteObject(hBrush)
            Call DeleteObject(hPen)
          End If
        Next N
      Case Is > 1
        For N = 1 To Depth
          For T = 1 To nSegments
            ToOffset = FromOffset + Segments(T).Value

            If Segments(T).Visible And Segments(T).Value > 0 Then
              FromWinkel = m_Angle + FromOffset / Max * 2 * Pi
              ToWinkel = m_Angle + ToOffset / Max * 2 * Pi

              StartPunkt.X = -Cos(FromWinkel) * Radius.X
              StartPunkt.Y = Sin(FromWinkel) * Radius.Y * IIf(ClockWise, -1, 1)
            
              EndPunkt.X = -Cos(ToWinkel) * Radius.X
              EndPunkt.Y = Sin(ToWinkel) * Radius.Y * IIf(ClockWise, -1, 1)
            
              Drift.X = -Cos((FromWinkel + ToWinkel) / 2) * IIf(Segments(T).Distant, m_LargeDistance, m_SmallDistance)
              Drift.Y = Sin((FromWinkel + ToWinkel) / 2) * IIf(Segments(T).Distant, m_LargeDistance, m_SmallDistance) * Height / Width * IIf(ClockWise, -1, 1)

              hPen = CreatePen(vbSolid, 1, MakeDarkColor(Segments(T).Color))
              hOldPen = SelectObject(hDC, hPen)

              hBrush = CreateSolidBrush(Segments(T).Color)
              hOldBrush = SelectObject(hDC, hBrush)

              If ClockWise Then
                Call Pie(hDC, X + Drift.X, Y + Drift.Y - N, X + Width + Drift.X, Y + Height + Drift.Y - N, X + Radius.X + EndPunkt.X + Drift.X, Y + Radius.Y + EndPunkt.Y + Drift.Y - N, X + Radius.X + StartPunkt.X + Drift.X, Y + Radius.Y + StartPunkt.Y + Drift.Y - N)
              Else
                Call Pie(hDC, X + Drift.X, Y + Drift.Y - N, X + Width + Drift.X, Y + Height + Drift.Y - N, X + Radius.X + StartPunkt.X + Drift.X, Y + Radius.Y + StartPunkt.Y + Drift.Y - N, X + Radius.X + EndPunkt.X + Drift.X, Y + Radius.Y + EndPunkt.Y + Drift.Y - N)
              End If

              Call SelectObject(hDC, hOldBrush)
              Call SelectObject(hDC, hOldPen)
              Call DeleteObject(hBrush)
              Call DeleteObject(hPen)
            End If

            FromOffset = FromOffset + Segments(T).Value
          Next T
        Next N
    End Select
  End If
End Sub

'*** Weitere Eigenschaften der Klasse

'----------------------------------------------------
' SegmentValue
' set or retrieves the value of a segment
' - IN Index As Long: index of the segment
'----------------------------------------------------
Public Property Get SegmentValue(ByVal Index As Long) As Long
  If Index > 0 And Index <= nSegments Then
    SegmentValue = Segments(Index).Value
  End If
End Property

Public Property Let SegmentValue(ByVal Index As Long, ByVal NewValue As Long)
  If Index > 0 And Index <= nSegments Then
    Segments(Index).Value = NewValue
  End If

  Call CheckMax
End Property

'----------------------------------------------------
' SegmentColor
' set or retrieves the color of a segment
' - IN Index As Long: index of the segment
'----------------------------------------------------
Public Property Get SegmentColor(ByVal Index As Long) As Long
  If Index > 0 And Index <= nSegments Then
    SegmentColor = Segments(Index).Color
  End If
End Property

Public Property Let SegmentColor(ByVal Index As Long, ByVal NewColor As Long)
  If Index > 0 And Index <= nSegments Then
    Segments(Index).Color = NewColor
  End If
End Property

'----------------------------------------------------
' SegmentValue
' set or retrieves the visibility of a segment
' - IN Index As Long: index of the segment
'----------------------------------------------------
Public Property Get SegmentVisible(ByVal Index As Long) As Boolean
  If Index > 0 And Index <= nSegments Then
    SegmentVisible = Segments(Index).Visible
  End If
End Property

Public Property Let SegmentVisible(ByVal Index As Long, ByVal NewVisible As Boolean)
  If Index > 0 And Index <= nSegments Then
    Segments(Index).Visible = NewVisible
  End If
End Property

'----------------------------------------------------
' SegmentDistant
' specifies weather a segment is more distant from the center
' - IN Index As Long: index of the segment
'----------------------------------------------------
Public Property Get SegmentDistant(ByVal Index As Long) As Boolean
  If Index > 0 And Index <= nSegments Then
    SegmentDistant = Segments(Index).Distant
  End If
End Property

Public Property Let SegmentDistant(ByVal Index As Long, ByVal NewDistant As Boolean)
  If Index > 0 And Index <= nSegments Then
    Segments(Index).Distant = NewDistant
  End If
End Property

'----------------------------------------------------
' OffsetAngle
' set or retrieves the offset angle
' - IN AngleSystem As AngleSystemConstants: (optional) specifies the used angle system
'----------------------------------------------------
Public Property Get OffsetAngle(Optional ByVal AngleSystem As AngleSystemConstants = asRAD) As Double
  Select Case AngleSystem
    Case AngleSystemConstants.asRAD
      OffsetAngle = m_Angle
    Case AngleSystemConstants.asDEG
      OffsetAngle = (m_Angle / Pi) * 180
  End Select
End Property

Public Property Let OffsetAngle(Optional ByVal AngleSystem As AngleSystemConstants = asRAD, ByVal NewAngle As Double)
  Select Case AngleSystem
    Case AngleSystemConstants.asRAD
      m_Angle = NewAngle
    Case AngleSystemConstants.asDEG
      m_Angle = (NewAngle / 180) * Pi
  End Select
End Property

'----------------------------------------------------
' SmallDistance
' set or retrieves the distance of a normal segment
'----------------------------------------------------
Public Property Get SmallDistance() As Long
  SmallDistance = m_SmallDistance
End Property

Public Property Let SmallDistance(ByVal NewSmallDistance As Long)
  If NewSmallDistance < 0 Then NewSmallDistance = 0
  
  m_SmallDistance = NewSmallDistance

  If m_LargeDistance < m_SmallDistance Then m_LargeDistance = m_SmallDistance
End Property

'----------------------------------------------------
' LargeDistance
' set or retrieves the distance of a distant segment
'----------------------------------------------------
Public Property Get LargeDistance() As Long
  LargeDistance = m_LargeDistance
End Property

Public Property Let LargeDistance(ByVal NewLargeDistance As Long)
  If NewLargeDistance < 0 Then NewLargeDistance = 0
  
  m_LargeDistance = NewLargeDistance
  
  If m_LargeDistance < m_SmallDistance Then m_LargeDistance = m_SmallDistance
End Property

'----------------------------------------------------
' Depth
' set or retrieves the depth of the pie chart
'----------------------------------------------------
Public Property Get Depth() As Long
  Depth = m_Depth
End Property

Public Property Let Depth(ByVal NewDepth As Long)
  If NewDepth < 1 Then NewDepth = 1
  
  m_Depth = NewDepth
End Property

'*** internal functions

'----------------------------------------------------
' CheckMax
' summarize all segment values
'----------------------------------------------------
Private Sub CheckMax()
  Dim T As Long
  Dim NewMax As Long
  
  For T = 1 To nSegments
    NewMax = NewMax + Segments(T).Value
  Next T
  
  Max = NewMax
End Sub

'----------------------------------------------------
' MakeDarkColor
' darkens a color by removing 50% of each rgb value
' - IN Color As Long: color to be darkened
'----------------------------------------------------
Private Function MakeDarkColor(Color As Long) As Long
  Dim r As Integer
  Dim g As Integer
  Dim b As Integer
  
  b = (Color And &HFF0000) / &H10000
  g = (Color And &HFF00&) / &H100&
  r = Color And &HFF&

  MakeDarkColor = RGB(CInt(r / 2), CInt(g / 2), CInt(b / 2))
End Function

'*** set default values

Private Sub Class_Initialize()
  Depth = 1
End Sub
